import 'dart:collection';
import 'package:flutter/material.dart';
import 'package:rapid_widgets_library/widget-classesUsed.dart';
import 'custom-icon.dart';

class ImageTextCard {
  Color shadowColor;
  Color backgroundColor;
  double cardWidth;
  String imagePath;
  Map keyMap;
  double elevation;
  double borderRadius;
  double textPadding;
  dynamic apiData;
}

buildCard(ImageTextCard object) {
  var widgetList = buildTextWidget(object.keyMap, object.apiData);
  return Container(
    width: object.cardWidth,
    child: Card(
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(object.borderRadius),
        ),
        elevation: object.elevation,
        shadowColor: object.shadowColor,
        child: Column(children: <Widget>[
          object.imagePath.isNotEmpty ? Container(child: Image.network(object.imagePath)) : Container(),
          Container(
            color: object.backgroundColor,
            child: Padding(
              padding: EdgeInsets.all(object.textPadding),
              child: Column(
                children: widgetList,
              ),
            ),
          )
        ])),
  );
}

List<Widget> buildTextWidget(Map keyValueMap, apiData) {
  List<Widget> textList = [];

  keyValueMap = convertTextKeysToInt(keyValueMap); //need to find a way to remove this function
  for (int j = 0; j < keyValueMap.length; j++) {
    if (keyValueMap[j] != null) {
      String key = "";
      String iconName = "";
      IconState icon;
      //print(keyValueMap[j]);
      if (keyValueMap[j]["key"].toString().isNotEmpty) key = keyValueMap[j]["key"];

      if (keyValueMap[j]["icon"].toString().isNotEmpty && keyValueMap[j]["icon"] != "Select") {
        iconName = keyValueMap[j]["icon"];
        icon = new IconState(icon: fetchIcon(iconName), color: convertStringToColor(keyValueMap[j]["color"]), size: double.parse(keyValueMap[j]["fontSize"]) + 2);
      }

      var pathValue = getDataFromAPIPath(keyValueMap[j]["keyPath"], apiData);
      //print(pathValue);
      textList.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Align(
          alignment: Alignment.center,
          child: Row(children: [
            icon == null
                ? Container()
                : Padding(
                    padding: const EdgeInsets.only(right: 8.0),
                    child: customisedIcon(icon),
                  ),
            Expanded(
                child: Wrap(
                    children: [Text(key + pathValue.toString(), softWrap: true, style: TextStyle(fontSize: double.parse(keyValueMap[j]["fontSize"]), color: convertStringToColor(keyValueMap[j]["color"]), fontFamily: keyValueMap[j]["fontFamily"]))]))
          ]),
        ),
      ));
    }
  }
  return textList;
}

IconData fetchIcon(String name) {
  IconData icon = IconData(58917, fontFamily: 'MaterialIcons');

  if (name == 'add') {
    return Icons.add;
  } else if (name == 'home') {
    return Icons.home;
  } else if (name == 'account_box') {
    return Icons.account_box;
  } else if (name == 'email') {
    return Icons.email;
  } else if (name == 'location') {
    return Icons.location_city;
  } else if (name == 'flag') {
    return Icons.flag;
  } else if (name == 'expand') {
    return Icons.expand;
  } else if (name == 'more') {
    return Icons.more_horiz;
  } else if (name == 'options') {
    return Icons.more_vert;
  } else if (name == 'moving') {
    return Icons.moving;
  }
  return null;
}

Color convertStringToColor(String colorCode) {
  String valueString = colorCode.split('(0x')[1].split(')')[0];
  return Color(int.parse(valueString, radix: 16));
}

Map convertTextKeysToInt(Map keyValueMap) {
  Map<int, dynamic> returnMap = new HashMap();
  for (var key in keyValueMap.keys) {
    returnMap.putIfAbsent(int.parse(key.toString()), () => keyValueMap[key]);
  }
  return returnMap;
}

getDataFromAPIPath(parameterPath, jsonData) {
  var data = jsonData;
  var path = getPathForAPI(parameterPath);
  for (var item in path) {
    final number = num.tryParse(item);
    if (number == null) {
      if (data.containsKey(item))
        data = data[item.toString()];
      else
        data = "";
    } else {
      data = data[int.parse(item.toString())];
    }
  }

  return data;
}

List<dynamic> getPathForAPI(String path) {
  List<dynamic> pathArray = path.split("/");
  return pathArray;
}
